# Generated by Django 5.2.4 on 2025-08-04 19:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0034_aspathlist_routingpolicyrule_match_aspath_list_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='aspathlist',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='aspathlistrule',
            options={'ordering': ('aspath_list', 'index')},
        ),
        migrations.AlterModelOptions(
            name='bgppeergroup',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='bgpsession',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='community',
            options={'ordering': ['value']},
        ),
        migrations.AlterModelOptions(
            name='communitylist',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='communitylistrule',
            options={'ordering': ['community_list', 'community']},
        ),
        migrations.AlterModelOptions(
            name='prefixlist',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='prefixlistrule',
            options={'ordering': ['prefix_list', 'index']},
        ),
        migrations.AlterModelOptions(
            name='routingpolicy',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='routingpolicyrule',
            options={'ordering': ['routing_policy', 'index']},
        ),
    ]



from utilities.testing import ViewTestCases
from ipam.models import Prefix

from netbox_bgp.models import (
    ASPathList,
    ASPathListRule,
    PrefixList,
    PrefixListRule,
    RoutingPolicy,
    RoutingPolicyRule,
    Community,
    CommunityList
)


class ASPathListRuleTestCase(ViewTestCases.BulkImportObjectsViewTestCase):
    model = ASPathListRule

    def _get_base_url(self):
        return 'plugins:{}:{}_{{}}'.format(
            self.model._meta.app_label,
            self.model._meta.model_name
        )

    @classmethod
    def setUpTestData(cls):
        aspathlists = [
            ASPathList(name='aspathlist1'),
            ASPathList(name='aspathlist2'),
        ]
        ASPathList.objects.bulk_create(aspathlists)
        aspathlistrules = [
            ASPathListRule(aspath_list=aspathlists[0],index=5,action='permit',pattern='65000')
        ]
        ASPathListRule.objects.bulk_create(aspathlistrules)
        cls.csv_data = (
            "aspath_list,index,action,pattern",
            f"{aspathlists[0].name},10,permit,65001",
            f"{aspathlists[1].name},20,permit,65002",
        )
        cls.csv_update_data = (
            "id,aspath_list,index,action,pattern",
            f"{aspathlistrules[0].pk},{aspathlists[0].name},5,deny,65000",
        )

class PrefixListRuleTestCase(ViewTestCases.BulkImportObjectsViewTestCase):
    model = PrefixListRule

    def _get_base_url(self):
        return 'plugins:{}:{}_{{}}'.format(
            self.model._meta.app_label,
            self.model._meta.model_name
        )

    @classmethod
    def setUpTestData(cls):
        prefixes = [
            Prefix(prefix='1.1.1.0/24'),
            Prefix(prefix='2.2.2.0/24')
        ]
        Prefix.objects.bulk_create(prefixes)
        prefixlists = [
            PrefixList(name='prefixlist1'),
            PrefixList(name='prefixlist2'),
        ]
        PrefixList.objects.bulk_create(prefixlists)
        prefixlistrules = [
            PrefixListRule(prefix_list=prefixlists[0], index=5, action='permit', prefix=prefixes[0])
        ]
        PrefixListRule.objects.bulk_create(prefixlistrules)
        cls.csv_data = (
            "prefix_list,index,action,prefix",
            f"{prefixlists[0].name},10,permit,{prefixes[0].prefix}",
            f"{prefixlists[1].name},10,permit,{prefixes[1].prefix}",
        )
        cls.csv_update_data = (
            "id,prefix_list,index,action",
            f"{prefixlistrules[0].pk},{prefixlists[0].name},5,deny",
        )

class RoutingPolicyRuleTestCase(ViewTestCases.BulkImportObjectsViewTestCase):
    model = RoutingPolicyRule

    def _get_base_url(self):
        return 'plugins:{}:{}_{{}}'.format(
            self.model._meta.app_label,
            self.model._meta.model_name
        )

    @classmethod
    def setUpTestData(cls):
        aspathlists = [
            ASPathList(name='aspathlist1'),
        ]
        ASPathList.objects.bulk_create(aspathlists)
        prefixlists = [
            PrefixList(name='prefixlist1'),
            PrefixList(name='prefixlist2'),
        ]
        PrefixList.objects.bulk_create(prefixlists)
        rps = [
            RoutingPolicy(name='rp1'),
            RoutingPolicy(name='rp2'),
        ]
        RoutingPolicy.objects.bulk_create(rps)
        communities = [
            Community(value='65001:1')
        ]
        Community.objects.bulk_create(communities)
        comm_lists = [
            CommunityList(name='cl1')
        ]
        CommunityList.objects.bulk_create(comm_lists)
        rp_rules = [
            RoutingPolicyRule(routing_policy=rps[0], index=10, action='permit')
        ]
        RoutingPolicyRule.objects.bulk_create(rp_rules)

        cls.csv_data = (
            "routing_policy,index,action,match_community,match_community_list,match_aspath_list,match_ip_address,match_ipv6_address",
            f"{rps[0].name},20,permit,{communities[0].value},{comm_lists[0].name},{aspathlists[0].name},{prefixlists[0].name},{prefixlists[1].name}",
            f"{rps[1].name},10,permit,{communities[0].value},{comm_lists[0].name},{aspathlists[0].name},{prefixlists[0].name},{prefixlists[1].name}",
        )
        cls.csv_update_data = (
            "id,routing_policy,index,action",
            f"{rp_rules[0].pk},{rps[0].name},10,deny",
        )



from dcim.models import Device, Interface, Site
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q, QuerySet
from django.http import HttpRequest
from ipam.models import ASN, IPAddress
from netbox.plugins import PluginTemplateExtension
from netbox.views import generic
from tenancy.models import Tenant
from utilities.views import ViewTab, register_model_view
from virtualization.models import VirtualMachine

from .filtersets import BGPSessionFilterSet
from .models import BGPSession
from .tables import BGPSessionTable

# Load configuration
config = getattr(settings, "PLUGINS_CONFIG", {}).get("netbox_bgp", {})

# Only register the tab view if configuration is set to 'tab'
if config.get("device_ext_page", "") == "tab":

    @register_model_view(Device, name="bgp_sessions", path="bgp-sessions")
    class DeviceBGPSessionTabView(generic.ObjectChildrenView):
        """View to display BGP sessions for this device in a tab."""

        queryset = Device.objects.all()
        child_model = BGPSession
        filterset = BGPSessionFilterSet
        table = BGPSessionTable
        template_name = "generic/object_children.html"
        hide_if_empty = False
        tab = ViewTab(
            label="BGP Sessions",
            badge=lambda obj: BGPSession.objects.filter(device=obj).count(),
            permission="netbox_bgp.view_bgpsession",
        )

        def get_children(self, request, parent):
            """Get BGP sessions for this device."""
            return BGPSession.objects.filter(device=parent)


@register_model_view(IPAddress, name="bgp-sessions", path="bgp-sessions")
class IPAddressBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with an IP address."""

    queryset = IPAddress.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    @staticmethod
    def _get_ip_bgp_sessions(ip_address: IPAddress) -> QuerySet[BGPSession]:
        """Helper to get BGP sessions related to an IP address."""
        return BGPSession.objects.filter(
            Q(local_address=ip_address) | Q(remote_address=ip_address)
        ).distinct()

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: IPAddressBGPSessionsView._get_ip_bgp_sessions(obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(
        self, request: HttpRequest, parent: IPAddress
    ) -> QuerySet[BGPSession]:
        """Get BGP sessions where the IP address is either the local or remote address."""
        return IPAddressBGPSessionsView._get_ip_bgp_sessions(parent)


@register_model_view(Site, name="bgp-sessions", path="bgp-sessions")
class SiteBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with a site."""

    queryset = Site.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: BGPSession.objects.filter(site=obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(self, request: HttpRequest, parent: Site) -> QuerySet[BGPSession]:
        """Get BGP sessions for the site."""
        return BGPSession.objects.filter(site=parent)


@register_model_view(Tenant, name="bgp-sessions", path="bgp-sessions")
class TenantBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with a tenant."""

    queryset = Tenant.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: BGPSession.objects.filter(tenant=obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(
        self, request: HttpRequest, parent: Tenant
    ) -> QuerySet[BGPSession]:
        """Get BGP sessions for the tenant."""
        return BGPSession.objects.filter(tenant=parent)


@register_model_view(VirtualMachine, name="bgp-sessions", path="bgp-sessions")
class VirtualMachineBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with a virtual machine."""

    queryset = VirtualMachine.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: BGPSession.objects.filter(virtualmachine=obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(
        self, request: HttpRequest, parent: VirtualMachine
    ) -> QuerySet[BGPSession]:
        """Get BGP sessions for the virtual machine."""
        return BGPSession.objects.filter(virtualmachine=parent)


@register_model_view(ASN, name="bgp-sessions", path="bgp-sessions")
class ASNBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with an ASN."""

    queryset = ASN.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    @staticmethod
    def _get_asn_bgp_sessions(asn: ASN) -> QuerySet[BGPSession]:
        """Helper to get BGP sessions related to an ASN."""
        return BGPSession.objects.filter(Q(local_as=asn) | Q(remote_as=asn)).distinct()

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: ASNBGPSessionsView._get_asn_bgp_sessions(obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(self, request: HttpRequest, parent: ASN) -> QuerySet[BGPSession]:
        """Get BGP sessions where the ASN is either the local or remote AS."""
        return ASNBGPSessionsView._get_asn_bgp_sessions(parent)


@register_model_view(Interface, name="bgp-sessions", path="bgp-sessions")
class InterfaceBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with an interface."""

    queryset = Interface.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    @staticmethod
    def _get_interface_bgp_sessions(interface: Interface) -> QuerySet[BGPSession]:
        """Helper to get BGP sessions related to an interface via its IP addresses."""
        ct = ContentType.objects.get_for_model(Interface)
        ips = IPAddress.objects.filter(
            assigned_object_type=ct, assigned_object_id=interface.pk
        )
        return BGPSession.objects.filter(
            Q(local_address__in=ips) | Q(remote_address__in=ips)
        ).distinct()

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: InterfaceBGPSessionsView._get_interface_bgp_sessions(
            obj
        ).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(
        self, request: HttpRequest, parent: Interface
    ) -> QuerySet[BGPSession]:
        """Get BGP sessions for the interface."""
        return InterfaceBGPSessionsView._get_interface_bgp_sessions(parent)


# Register only when device_ext_page is set to 'tab';
class DeviceBGPSessionsView(generic.ObjectChildrenView):
    """View to display BGP sessions associated with a device."""

    queryset = Device.objects.all()
    child_model = BGPSession
    filterset = BGPSessionFilterSet
    table = BGPSessionTable
    template_name = "generic/object_children.html"
    hide_if_empty = False

    tab = ViewTab(
        label="BGP Sessions",
        badge=lambda obj: BGPSession.objects.filter(device=obj).count(),
        permission="netbox_bgp.view_bgpsession",
    )

    def get_children(
        self, request: HttpRequest, parent: Device
    ) -> QuerySet[BGPSession]:
        """Get BGP sessions for the device."""
        return BGPSession.objects.filter(device=parent)


# Register the BGP sessions tab view only if device_ext_page is set to 'tab';
# otherwise, use the PluginTemplateExtension for inline display (left, right, full_width).
if settings.PLUGINS_CONFIG.get("netbox_bgp", {}).get("device_ext_page") == "tab":
    DeviceBGPSessionsView = register_model_view(
        Device, name="bgp-sessions", path="bgp-sessions"
    )(DeviceBGPSessionsView)


class DeviceBGPSession(PluginTemplateExtension):
    models = ("dcim.device",)

    def left_page(self):
        if self.context["config"].get("device_ext_page") == "left":
            return self.x_page()
        return ""

    def right_page(self):
        if self.context["config"].get("device_ext_page") == "right":
            return self.x_page()
        return ""

    def full_width_page(self):
        if self.context["config"].get("device_ext_page") == "full_width":
            return self.x_page()
        return ""

    def x_page(self):
        return self.render(
            "netbox_bgp/device_extend.html",
        )


template_extensions = [DeviceBGPSession]



# Generated by Django 4.2.8 on 2024-02-16 05:58

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0030_netbox_bgp_comments'),
    ]

    operations = [
        migrations.CreateModel(
            name='CommunityList',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name_plural': 'Community Lists',
                'unique_together': {('name', 'description')},
            },
        ),
        migrations.AlterField(
            model_name='community',
            name='value',
            field=models.CharField(max_length=64, validators=[django.core.validators.RegexValidator('[\\d\\.\\*]+:[\\d\\.\\*]+')]),
        ),
        migrations.CreateModel(
            name='CommunityListRule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('action', models.CharField(max_length=30)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('community', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='netbox_bgp.community')),
                ('community_list', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='commlistrules', to='netbox_bgp.communitylist')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='routingpolicyrule',
            name='match_community_list',
            field=models.ManyToManyField(blank=True, related_name='cmrules', to='netbox_bgp.communitylist'),
        ),
    ]



# Generated by Django 5.2.9 on 2026-01-20 18:18

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("netbox_bgp", "0037_alter_bgpsession_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="bgpsession",
            name="max_prefixes",
            field=models.PositiveIntegerField(
                blank=True,
                null=True,
                validators=[django.core.validators.MinValueValidator(1)],
            ),
        ),
    ]



# Generated by Django 5.2.5 on 2025-09-29 20:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0035_alter_aspathlist_options_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='routingpolicy',
            options={'ordering': ['weight', 'name']},
        ),
        migrations.AddField(
            model_name='routingpolicy',
            name='weight',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.2.9 on 2025-12-16 23:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0036_alter_routingpolicy_options_routingpolicy_weight'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bgpsession',
            options={'ordering': ('name', 'pk')},
        ),
    ]



# Generated by Django 5.2.9 on 2026-02-13 13:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("netbox_bgp", "0038_bgpsession_max_prefix"),
    ]

    operations = [
        migrations.AddField(
            model_name="bgpsession",
            name="remote_as_macro",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
